@@ -1,5 +1,30 @@
 package Perl::Dist::WiX::DirectoryTree2;
 
+=pod
+
+=head1 NAME
+
+Perl::Dist::WiX::DirectoryTree2 - Base directory tree for Perl::Dist::WiX.
+
+=head1 VERSION
+
+This document describes Perl::Dist::WiX::DirectoryTree2 version 1.200_100.
+
+=head1 SYNOPSIS
+
+	$tree = Perl::Dist::WiX::DirectoryTree2->instance();
+	
+	# See each method for examples.
+
+=head1 DESCRIPTION
+
+This is an object that represents the main directory tree for the 
+installer.  This tree contains all directories being created that
+are referenced in more than one fragment, and all directories that
+need to have specific IDs.
+
+=cut
+
 use 5.008001;
 
 #use metaclass (
@@ -11,34 +36,71 @@ use MooseX::Singleton;
 use Params::Util qw( _IDENTIFIER _STRING _INSTANCE );
 use File::Spec::Functions qw( catdir catpath splitdir splitpath );
 use MooseX::Types::Moose qw( Str );
+use MooseX::Types::Path::Class qw( Dir );
 use Perl::Dist::WiX::Tag::Directory;
 use WiX3::Exceptions;
 
-our $VERSION = '1.102_102';
+our $VERSION = '1.200_100';
 $VERSION =~ s/_//sm;
 
 with 'WiX3::Role::Traceable';
 
-#####################################################################
-# Accessors:
-#   root: Returns the root of the directory tree created by new.
+=head1 METHODS
 
-has root => (
-	is     => 'ro',
-	isa    => 'Perl::Dist::WiX::Tag::Directory',
-	reader => 'get_root',
-	handles =>
-	  [qw(search_dir get_directory_object _add_directory_recursive)],
+=head2 new
+
+	my $tree = Perl::Dist::WiX::DirectoryTree2->new(
+		app_dir => 'C:\strawberry',
+		app_name => 'Strawberry Perl'
+	);
+
+Creates new directory tree object and creates the 'root' of the tree.
+
+Note that this object is a L<MooseX::Singleton|MooseX::Singleton> object,
+so that you can retrieve the object at any time using the 
+C<instance()> method.
+
+=cut
+
+# This is private, but retrievable by 'get_root'.
+has _root => (
+	is       => 'bare',
+	isa      => 'Perl::Dist::WiX::Tag::Directory',
+	reader   => 'get_root',
 	required => 1,
+	handles  => {
+		'search_dir'               => 'search_dir',
+		'get_directory_object'     => 'get_directory_object',
+		'_add_directory_recursive' => '_add_directory_recursive',
+		'_indent'                  => 'indent',
+	},
 );
 
+
+=head3 app_dir
+
+This is set to the distribution's image_dir (where the distribution is
+going to be installed by default.) 
+
+=cut
+
+
 has app_dir => (
 	is       => 'ro',
-	isa      => Str,
+	isa      => Dir,
 	reader   => '_get_app_dir',
 	required => 1,
+	coerce   => 1,
 );
 
+
+=head3 app_name
+
+This is set to the name of the distribution, and is used to set the
+name of the Start Menu directory containing the distribution's icons.
+
+=cut
+
 has app_name => (
 	is       => 'ro',
 	isa      => Str,
@@ -71,31 +133,70 @@ sub BUILDARGS {
 		where     => 'Perl::Dist::WiX::DirectoryTree2->new'
 	  );
 
-	my $root = Perl::Dist::WiX::Tag::Directory->new(
-		id       => 'TARGETDIR',
-		name     => 'SourceDir',
-		path     => $app_dir,
-		noprefix => 1,
-	);
+	if ( exists $args{_root} ) {
 
-	return {
-		root => $root,
-		%args
-	};
+		# If we're recreating, the assumption is that
+		# we know what we're doing.
+		return \%args;
+	} else {
+
+		# Create the root directory object.
+		my $root = Perl::Dist::WiX::Tag::Directory->new(
+			id       => 'TARGETDIR',
+			name     => 'SourceDir',
+			path     => "$app_dir",
+			noprefix => 1,
+		);
+
+		return {
+			_root => $root,
+			%args
+		};
+	} ## end else [ if ( exists $args{_root...})]
 } ## end sub BUILDARGS
 
+=head2 instance
+
+	my $tree = Perl::Dist::WiX::DirectoryTree2->instance();
+	
+Returns the previously created directory tree. 
+
+=head2 get_root
+
+	my $directory_object = $tree->get_root();
+	
+Gets the L<Perl::Dist::WiX::Tag::Directory|Perl::Dist::WiX::Tag::Directory> 
+object at the root of the tree.
+	
+=head2 as_string
+
+	my $string = $tree->as_string();
+	
+This method returns an XML representation of the directory tree.
+
+=cut
+
 sub as_string {
 	my $self = shift;
 
 	my $string = $self->get_root()->as_string();
 
-	return $string ne q{} ? $self->get_root()->indent( 4, $string ) : q{};
+	return $string ne q{} ? $self->_indent( 4, $string ) : q{};
 }
 
+=head2 initialize_tree
+
+	$tree->initialize_tree($perl_version, $bits, $gcc_version);
+
+Adds a basic directory structure to the directory tree object.
+
+=cut
 
 sub initialize_tree {
 	my $self = shift;
 	my $ver  = shift;
+	my $bits = shift || 32;
+	my $gcc  = shift || 3;
 
 	$self->trace_line( 2, "Initializing directory tree.\n" );
 
@@ -103,9 +204,9 @@ sub initialize_tree {
 	my $branch = $self->get_root()->add_directory( {
 			id       => 'INSTALLDIR',
 			noprefix => 1,
-			path     => $self->_get_app_dir(),
+			path     => $self->_get_app_dir()->stringify(),
 		} );
-	$self->get_root()->add_directory( {
+	my $app_menu = $self->get_root()->add_directory( {
 			id       => 'ProgramMenuFolder',
 			noprefix => 1,
 		}
@@ -115,6 +216,11 @@ sub initialize_tree {
 		} );
 
 #<<<
+	$app_menu->add_directories_id(
+		'App_Menu_Tools',    'Tools',
+		'App_Menu_Websites', 'Related Websites',
+	);
+
 	$branch->add_directories_id(
 		'Perl',      'perl',
 		'Toolchain', 'c',
@@ -152,13 +258,37 @@ sub initialize_tree {
 	  perl\\vendor\\lib\\auto\\share\\module
 	);
 
+# We have to get every possibility of directories immediately under
+# the 'c' directory, or linking errors occur, as c is found first in later files.
+	if ( 64 == $bits ) {
+		push @list, 'c\\lib64';
+		push @list, 'c\\x86_64-w64-mingw32';
+	}
+
 	foreach my $dir (@list) {
-		$self->add_directory( catdir( $self->_get_app_dir(), $dir ) );
+		$self->add_directory(
+			$self->_get_app_dir()->subdir($dir)->stringify() );
 	}
 
 	return $self;
 } ## end sub initialize_tree
 
+
+
+=head2 initialize_short_tree
+
+	$tree->initialize_short_tree();
+
+Adds a basic directory structure to the directory tree object.
+
+This is used when including a merge module that already 
+contains a L<Perl::Dist::WiX|Perl::Dist::WiX>-based perl
+distribution.
+
+=cut
+
+
+
 sub initialize_short_tree {
 	my $self = shift;
 
@@ -168,9 +298,9 @@ sub initialize_short_tree {
 	my $branch = $self->get_root()->add_directory( {
 			id       => 'INSTALLDIR',
 			noprefix => 1,
-			path     => $self->_get_app_dir(),
+			path     => $self->_get_app_dir()->stringify(),
 		} );
-	$self->get_root()->add_directory( {
+	my $app_menu = $self->get_root()->add_directory( {
 			id       => 'ProgramMenuFolder',
 			noprefix => 1,
 		}
@@ -180,6 +310,11 @@ sub initialize_short_tree {
 		} );
 
 #<<<
+	$app_menu->add_directories_id(
+		'App_Menu_Tools',    'Tools',
+		'App_Menu_Websites', 'Related Websites',
+	);
+
 	$branch->add_directories_id(
 		'Win32',     'win32',
 		'Perl',      'perl',
@@ -192,6 +327,15 @@ sub initialize_short_tree {
 	return $self;
 } ## end sub initialize_short_tree
 
+=head2 add_directory
+
+	$tree->add_directory($directory);
+
+Adds a directory to the tree, including all directories required along 
+the way.
+	
+=cut
+
 sub add_directory {
 	my $self = shift;
 	my $dir  = shift;
@@ -229,6 +373,19 @@ sub add_directory {
 	return defined $dir_out ? 1 : 0;
 } ## end sub add_directory
 
+
+
+=head2 add_root_directory
+
+	$self->add_root_directory('Id', 'directory');
+
+Adds a directory entry with the ID and directory name given
+immediately under the main installation directory.
+	
+=cut
+
+
+
 sub add_root_directory {
 	my $self = shift;
 	my $id   = shift;
@@ -238,6 +395,20 @@ sub add_root_directory {
 	return $branch->add_directories_id( $id, $dir );
 }
 
+
+
+=head2 add_merge_module
+
+	$tree->add_merge_module('C:\strawberry', $mergemodule_object);
+
+This method inserts a merge module (referred to by a 
+L<Perl::Dist::WiX::Tag::MergeModule|Perl::Dist::WiX::Tag::MergeModule> 
+object) into the directory tree at the specified directory.
+
+=cut
+
+
+
 sub add_merge_module {
 	my $self = shift;
 	my $dir  = shift;
@@ -257,9 +428,11 @@ sub add_merge_module {
 
 	$directory_object->add_child_tag($mm);
 
-	return;
+	return 1;
 } ## end sub add_merge_module
 
+
+
 no Moose;
 __PACKAGE__->meta->make_immutable;
 
@@ -267,65 +440,6 @@ __PACKAGE__->meta->make_immutable;
 
 __END__
 
-=pod
-
-=head1 NAME
-
-Perl::Dist::WiX::DirectoryTree2 - Base directory tree for Perl::Dist::WiX.
-
-=head1 VERSION
-
-This document describes Perl::Dist::WiX::DirectoryTree2 version 1.090.
-
-=head1 DESCRIPTION
-
-This is an object that establishes a directory tree.
-
-=head1 METHODS
-
-=head2 new
-
-	my $tree = Perl::Dist::WiX::DirectoryTree2->new(
-		app_dir => 'C:\strawberry',
-		app_name => 'Strawberry Perl'
-	);
-
-Creates new directory tree object and creates the 'root' of the tree.
-
-Note that this object is a L<MooseX::Singleton|MooseX::Singleton> object.
-
-=head2 instance
-
-	my $tree = Perl::Dist::WiX::DirectoryTree2->instance();
-	
-Returns the previously created directory tree. 
-
-=head2 get_root
-
-	my $directory_object = $tree->get_root();
-	
-Gets the L<Perl::Dist::WiX::Directory|Perl::Dist::WiX::Directory> object at
-the root of the tree.
-	
-=head2 initialize_tree
-
-	$tree->initialize_tree($perl_version);
-
-Adds a basic directory structure to the directory tree object.
-
-=head2 add_directory
-
-	$tree->add_directory($directory);
-
-Adds a directory to the tree, including all directories required along 
-the way.
-	
-=head2 as_string
-
-	print $tree->as_string();
-	
-Prints out the tree as a series of XML tags.
-
 =head2 get_directory_object
 
 Calls L<Perl::Dist::WiX::Directory's get_directory_object routine|Perl::Dist::WiX::Directory/get_directory_object>
