@@ -5,16 +5,18 @@ use strict;
 use warnings;
 use File::Slurp qw(read_file write_file);
 use Getopt::Long qw(GetOptions);
+use Pod::Usage qw(pod2usage);
 use English qw( -no_match_vars );
 use File::Spec::Functions qw(splitpath catfile);
 use Carp qw(carp);
 use Win32::File::Object qw();
+use FindBin;
 
 sub usage;
 sub version;
 sub relocate_file;
 
-our $STRING_VERSION = our $VERSION = '1.000';
+our $STRING_VERSION = our $VERSION = '1.001';
 $VERSION  =~ s/_//;
 
 my @files;
@@ -31,12 +33,11 @@ GetOptions('help|?'     => sub { pod2usage(-exitstatus => 0, -verbose => 0); },
 		  ) or pod2usage(-verbose => 2);
 
 if (0 == scalar @files) {
-	@files = glob '*.reloc.txt';
+	@files = glob catfile($FindBin::Bin, '*.reloc.txt');
 }
 
 if (not defined $new_location) {
-	require Cwd;
-	$new_location = Cwd::cwd();
+	$new_location = $FindBin::Bin;
 	$new_location =~ s{/}{\\}g;
 }
 
@@ -211,7 +212,7 @@ Strawberry Perl to a new location.
 
 Usage: perl $script 
     [ --help ] [ --usage ] [ --man ] [ --version ] [ -? ]
-    --file relocationfile
+    [--file relocationfile] [--location path] [--quiet]
 
 For more assistance, run perl $script --help.
 EOF
@@ -249,7 +250,7 @@ This script updates all of Strawberry Perl's files to a new location.
     --file          Gives the location of the file of hints to use to 
                     relocate Perl. Defaults to all *.reloc.txt files in
                     the current directory.
-    --location      The location to relocate to. Defaults to Cwd::cwd().
+    --location      The location to relocate to. Defaults to $FindBin::Bin.
     --quiet         Print nothing.
 	
 =head1 DEPENDENCIES
